"use client"

import Image from "next/image"
import { useEffect } from "react"
import { useRouter } from "next/navigation"
import { toast } from "react-toastify"
import md5 from "js-md5"
import { execute } from "@/helper/clientApi"
import useI18n from "@/hooks/useI18n"
import useLoading from "@/hooks/useLoading"
import FormLogin from "@/components/Login/FormLogin"
import presets from "@/utils/globalPresets"
import environment from "@/utils/environment"
import useGlobalContainer from "@/hooks/useGlobalContainer"
import useHasMounted from "@/hooks/useHasMounted"

const LoginPage = () => {
  const global = useGlobalContainer()
  const router = useRouter()
  const i18n = useI18n()
  const loading = useLoading()
  const hasMounted = useHasMounted()
  const homePath = presets.locations.welcome

  const loginUsuario = async (datos) => {
    loading.start()
    if (datos && datos !== null) {
      const hash = md5.create()
      hash.update(datos.password.trim())
      try {
        const resultado = await execute("SPR_LOGIN", [datos.email.toLowerCase().trim(), hash.hex()])
        loading.stop()
        if (resultado === undefined) {
          toast.error(i18n.t("common.errorApi"), presets.toaster)
          return
        }
        if (resultado[0].status === "ERROR") {
          toast.error(resultado[0].message, presets.toaster)
        } else if (resultado[0].status === "WARNING") {
          toast.warning(resultado[0].message, presets.toaster)
        } else if (resultado[0].token) {
          const user = {
            email: resultado[0].correo_electronico,
            nombre_usuario: resultado[0].nombre_completo,
            // img_user: resultado[0].foto,
            token: resultado[0].token,
            es_admin: resultado[0].es_admin,
            compania: resultado[0].nombre_compania,
            departamento: resultado[0].nombre_departamento,
            tiempo_expiracion: resultado[0].tiempo_expiracion,
            fecha_inicio_sesion: resultado[0].fecha_inicio_sesion,
            fecha_fin_sesion: resultado[0].fecha_fin_sesion,
            theme: resultado[0].theme,
            i18n: process.env.defaultLocale
          }
          const constante = {
            apptitle: process.env.apptitle,
            max_mb_upload: process.env.maxMbUpload
          }
          await environment.login(user.token, user)
          toast.success(`Bienvenido ${user.nombre_usuario}`, presets.toaster)
          await global.setUserObj(user)
          await global.setConstanteObj(constante)
          await global.setToken(resultado[0].token)
          await global.setTheme(user.theme)
          await i18n.locale(user.i18n)
          router.push(homePath)
        }
      } catch (error) {
        toast.error(i18n.t("common.errorApi") + error, presets.toaster)
      }
    }
  }

  const getEnv = async () => {
    const env = await environment.getEnvUser()
    if (env && env.token) {
      router.push(presets.locations.welcome)
    }
  }

  // when page is mounted
  useEffect(() => {
    if (hasMounted) {
      getEnv()
    }
  }, [hasMounted])

  return (
    <div className={"via-login-page"}>
      <div className={"via-login-layout"}>
        {/* <!-- Tarjeta de Login --> */}
        <div className={"via-card"}>
          <div className={"via-login-header"}>
            <div className={"size-64 shrink-0"}>
              <Image src={presets.images.logo} alt="logo" className={"via-login-logo"} width={100} height={100} />
            </div>
          </div>
          <div className="align-center flex w-full p-4 ">
            <div className={"via-login-form"}>
              {/* <!-- Formulario --> */}
              <FormLogin onLogin={loginUsuario} />
              {/* <!-- Fin Formulario --> */}
            </div>
          </div>
        </div>
      </div>
      <div
        className={"via-login-background"}
        style={{ backgroundImage: `url(${presets.images.loginFondo})`, minHeight: "75vh", backgroundSize: "cover" }}
      />
    </div>
  )
}

export default LoginPage
