import * as React from "react"
import { useRouter } from "next/navigation"
import { execute } from "@/helper/clientApi"
import environment from "@/utils/environment"
import presets from "@/utils/globalPresets"
import dynamic from "next/dynamic"
import useLoading from "@/hooks/useLoading"
import { useStore } from "@/hooks/useStore"
import useHasMounted from "@/hooks/useHasMounted"
import useI18n from "@/hooks/useI18n"
import SessionTimeout from "@/components/SessionTimeout"
// import UserOptionsMenu from '@/components/widgets/UserOptionsMenu'
import { toast } from "react-toastify"
import { useTheme } from "next-themes"

export const LayoutContext = React.createContext()

const Footer = dynamic(() => { return import("vComponents/Footer") }, { ssr: false })
const Sidebar = dynamic(() => { return import("via-ui/sidebar") }, { ssr: false })
const Navbar = dynamic(() => { return import("vComponents/Navbar") }, { ssr: false })

const ResponsiveContainer = ({ children }) => {
  const router = useRouter()
  const loading = useLoading()
  const hasMounted = useHasMounted()
  const i18n = useI18n()
  const { theme, setTheme } = useTheme()

  const [sidebarOpen, setSidebarOpen] = React.useState(true)
  const [menu, setMenu] = React.useState([])
  const [title, setTitle] = useStore(s => s.title, a => a.setTitle)
  const [userObj, setUserObj] = React.useState()
  const [token, setToken] = React.useState("")
  const [constanteObj, setConstanteObj] = React.useState("")

  const doLogout = async () => {
    const redirectPath = await environment.logout()
    await i18n.locale("es")
    setUserObj(null)
    setConstanteObj(null)
    setTheme(null)
    router.push(redirectPath)
  }

  // se ejecuta cuando expira la sesion
  const onTimeout = () => {
    doLogout()
  }

  // se ejecuta cuando el usuario indica explicitamente que sigue conectado en la sesion
  const onCancelTimeout = async () => {
    const sprUpdateSesion = "SPR_SESION_U"
    const result = await execute(sprUpdateSesion, [token])
    if (environment.validaResultadoDB(result) === true) {
      const updateUserObj = { ...userObj }
      updateUserObj.tiempo_expiracion = result[0].tiempo_expiracion
      updateUserObj.fecha_fin_sesion = result[0].fecha_fin_sesion
      setUserObj(updateUserObj)
      await environment.login(token, updateUserObj)
    }
  }

  const setPreferences = async (opcion) => {
    const env = await environment.getEnvUser()
    // console.log((new Date()).toString(),'setPreferences userObj', userObj)
    const sprInfoSesion = "SPR_INFO_USUARIOS"
    const response = await execute(sprInfoSesion, [env.token, opcion])
    if (environment.validaResultadoDB(response, i18n, toast, false) === true) {
      // console.log('setPreferences', response)
      if (opcion === "en" || opcion === "es") {
        env.user.i18n = opcion
        // setLanguage(opcion)
      }
      if (opcion === "light" || opcion === "dark") {
        env.user.theme = opcion
        setTheme("light")
      }
      environment.setEnvUser(env.user)
      if (environment.validaResultadoDB(response, i18n, toast, false) === true) {
        if (response && response[0]) {
          toast.success(response[0].response, presets.toaster)
        }
      }
    }
  }

  // const validaPermisosRuta = async (token) => {
  //   if (router.pathname.includes('/admin')) {
  //     loading.start()
  //     const tienePermiso = await environment.validaPermisos(token, 'S', 'N', 'N', 'N')
  //     loading.stop()
  //     if (tienePermiso === false) {
  //       router.push(presets.locations.welcome)
  //     }
  //   }
  //   if (router.pathname.includes('/mantenimientos')) {
  //     loading.start()
  //     const tienePermiso = await environment.validaPermisos(token, 'N', 'S', 'N', 'N')
  //     loading.stop()
  //     if (tienePermiso === false) {
  //       router.push(presets.locations.welcome)
  //     }
  //   }
  // }

  const getMenu = async () => {
    loading.start()
    setMenu([])
    if (token && token !== null) {
      const options = await execute("SPR_MENU_S", [token, "BO", null])
      setMenu(options || [])
    }
    loading.stop()
  }

  const getEnv = async () => {
    try {
      const env = await environment.getEnvUser()

      if (!env || !env.token) {
        doLogout()
        return
      }
      setUserObj(env.user)
      setConstanteObj(env.constante)
      setToken(env.token)
      setTheme(env.user.theme || "light")
      if (!env.user.i18n) {
        const browserLanguage = window.navigator.language
        if (browserLanguage === "en" || browserLanguage === "en-US") {
          setPreferences("en")
        } else {
          setPreferences("es")
        }
      } else {
        setPreferences(env.user.idioma)
      }
    } catch (error) {
      doLogout()
    }
  }

  const containerWrapper = React.useMemo(() => ({
    userObj,
    setUserObj,
    constanteObj,
    setConstanteObj,
    token,
    setToken,
    setTheme,
    setTitle
  }), [userObj, setUserObj, constanteObj, setConstanteObj, token, setToken, setTheme, setTitle])

  React.useEffect(() => {
    getMenu()
  }, [token])

  // when page is mounted
  React.useEffect(() => {
    if (hasMounted) {
      getEnv().then(() => {
        getMenu()
      })
    }
  }, [hasMounted])

  return (
    <LayoutContext.Provider value={containerWrapper}>

      <div className={"flex h-screen overflow-hidden"}>
        {/* Sidebar */}
        {userObj && userObj.nombre_usuario &&
          <Sidebar
            sidebarOpen={sidebarOpen}
            setSidebarOpen={setSidebarOpen}
            menu={menu}
            presets={presets}
            userObj={userObj}
            environment={environment}
            setTitle={setTitle}
            onClickLogout={() => doLogout()}
            router={router}
            i18n={i18n}
          />
        }

        {/* Content area */}
        <div className={"relative flex w-full flex-1 flex-col overflow-y-auto overflow-x-hidden"}>

          {/*  Site header */}
          {userObj && userObj.nombre_usuario &&
            <Navbar
              sidebarOpen={sidebarOpen}
              setSidebarOpen={setSidebarOpen}
              onClickLogout={() => doLogout()}
              onClickProfile={() => router.push(`${presets.locations.profile}/${environment.getTime()}`)}
              // MenuOptions={() => <UserOptionsMenu setPreferences={setPreferences} theme={theme} i18n={i18n} />}
              i18n={i18n}
              theme={theme}
              setPreferences={setPreferences}
              title={title}
              userObj={userObj}
              router={router}
              presets={presets}
            />
          }

          <main className={userObj && userObj.nombre_usuario ? "responsive-container" : ""}>
            {children}
          </main>

          {userObj && userObj.nombre_usuario &&
            <SessionTimeout i18n={i18n} user={userObj} onTimeout={() => onTimeout()} onCancelTimeout={() => onCancelTimeout()} />
          }
        </div>

        {/* Footer */}
        <Footer version={process.env.version} label={i18n.t("common.version") || "version"} />
      </div>
    </LayoutContext.Provider>
  )
}

export default ResponsiveContainer
